#!/usr/bin/env node

/* eslint no-console:0 */

require('dotenv').config();
var usageReport = require('../');
var async = require('async');
var chalk = require('chalk');
var yargs = require('yargs');
var helpers = require('../lib/helpers.js');

var options = yargs.usage('Usage: $0 [reports]')
  .option('report', {
    alias: 'r',
    describe: 'specific reports to see',
    choices: ['session', 'launches', 'sampling', 'instance', 'update', 'users'],
    default: ['session', 'launches', 'sampling', 'instance', 'update', 'users']
  })
  .array('report')
  .help('help')
  .alias('help', 'h')
  .argv;

function asyncOsLaunches(callback) {
  var arr = [];
  var str1 = usageReport.getLaunchesOsData(process.env.MIX_SECRET);
  str1.on('data', function(data) {
    arr.push({
      date: data.date,
      'Mac OS X': data.values['Mac OS X'],
      'Windows': data.values.Windows
    });
  });
  str1.on('end', function() {
    var table = helpers.makeTable(['date', 'Mac OS X', 'Windows'], arr);
    callback(null, table.toString());
  });
}

function asyncVersionLaunches(osLaunches, callback) {
  var arr = [];
  var str1 = usageReport.getLaunchesVersionData(process.env.MIX_SECRET);
  var versions = [];
  str1.on('data', function(data) {
    var obj = {};
    obj.date = data.date;
    for (var vers in data.values) {
      if (data.values.hasOwnProperty(vers)) {
        obj[vers] = data.values[vers];
      }
      if (!helpers.isInArray(vers, versions)) {
        versions.push(vers);
      }
    }
    arr.push(obj);
  });
  str1.on('end', function() {
    versions.sort().unshift('date');
    var table = helpers.makeTable(versions, arr);
    callback(null, osLaunches, table.toString());
  });
}

function asyncSessionData(osLaunches, versionLaunches, callback) {
  var str = usageReport.getSessionData(process.env.MIX_SECRET);
  var userDict = {};
  str.on('data', function(chunk) {
    for (var user in chunk.values) {
      if (chunk.values.hasOwnProperty(user)) {
        if (userDict[user] === undefined) {
          userDict[user] = {
            days: 1,
            sessions: chunk.values[user].sessions,
            lengths: chunk.values[user].lengths
          };
        } else {
          userDict[user].days++;
          userDict[user].sessions += chunk.values[user].sessions;
          userDict[user].lengths += chunk.values[user].lengths;
        }
      }
    }
  });
  str.on('end', function() {
    var obj = {
      days: {
        property: 'days',
        values: []
      },
      sessions: {
        property: 'sessions',
        values: []
      },
      lengths: {
        property: 'lengths',
        values: []
      }
    };
    for (var user in userDict) {
      if (userDict.hasOwnProperty(user)) {
        obj.days.values.push(userDict[user].days);
        obj.sessions.values.push(userDict[user].sessions);
        obj.lengths.values.push(userDict[user].lengths);
      }
    }
    var arr = [];
    for (var key in obj) {
      if (obj.hasOwnProperty(key)) {
        var row = helpers.getStats(obj[key].values);
        row.property = obj[key].property;
        arr.push(row);
      }
    }
    var table = helpers.makeTable(['property', 'max', 'avg', 'med', 25, 75], arr);
    callback(null, osLaunches, versionLaunches, table.toString());
  });
}

function asyncSamplingData(osLaunches, versionLaunches, session, callback) {
  var str = usageReport.getSamplingData(process.env.MIX_SECRET);
  var arr = [];
  str.on('data', function(data) {
    arr = arr.concat(data.values);
  });
  str.on('end', function() {
    var obj = helpers.getStats(arr);
    obj.property = 'sampling';
    var table = helpers.makeTable(['property', 'max', 'avg', 'med', 25, 75], [obj]);
    callback(null, osLaunches, versionLaunches, session, table.toString());
  });
}

function asyncInstanceData(osLaunches, versionLaunches, session, sampling, callback) {
  var arr = [];
  var str = usageReport.getInstanceData(process.env.MIX_SECRET);
  var obj = {};
  var i;
  str.on('data', function(data) {
    if (obj.analysis === undefined) {
      obj.analysis = data.values.analysis;
      obj.steps = data.values.steps;
    } else {
      obj.analysis.starting_amount += data.values.analysis.starting_amount;
      obj.analysis.completion += data.values.analysis.completion;
      for (i = 0; i < data.values.steps.length; ++i) {
        obj.steps[i].count += data.values.steps[i].count;
      }
    }
  });
  str.on('end', function() {
    var started = obj.steps[0].count;
    for (i = 0; i < obj.steps.length; ++i) {
      arr.push({
        step: obj.steps[i].event,
        started: started,
        completed: obj.steps[i].count,
        conversion: (obj.steps[i].count / started).toFixed(2)
      });
      started = obj.steps[i].count;
    }
    arr.push({
      step: 'total',
      started: obj.analysis.starting_amount,
      completed: obj.analysis.completion,
      conversion: (obj.analysis.completion / obj.analysis.starting_amount).toFixed(2)
    });
    var table = helpers.makeTable(['step', 'started', 'completed', 'conversion'], arr);
    callback(null, osLaunches, versionLaunches, session, sampling, table.toString());
  });
}

function asyncUpdateData(osLaunches, versionLaunches, session, sampling, instance, callback) {
  var arr = [];
  var str = usageReport.updateSuccessData(process.env.MIX_SECRET);
  var obj = {};
  var i;
  str.on('data', function(data) {
    if (obj.analysis === undefined) {
      obj.analysis = data.values.analysis;
      obj.steps = data.values.steps;
    } else {
      obj.analysis.starting_amount += data.values.analysis.starting_amount;
      obj.analysis.completion += data.values.analysis.completion;
      for (i = 0; i < data.values.steps.length; ++i) {
        obj.steps[i].count += data.values.steps[i].count;
      }
    }
  });
  str.on('end', function() {
    var started = obj.steps[0].count;
    for (i = 0; i < obj.steps.length; ++i) {
      arr.push({
        step: obj.steps[i].event,
        started: started,
        completed: obj.steps[i].count,
        conversion: (obj.steps[i].count / started).toFixed(2)
      });
      started = obj.steps[i].count;
    }
    arr.push({
      step: 'total',
      started: obj.analysis.starting_amount,
      completed: obj.analysis.completion,
      conversion: (obj.analysis.completion / obj.analysis.starting_amount).toFixed(2)
    });
    var table = helpers.makeTable(['step', 'started', 'completed', 'conversion'], arr);
    callback(null, osLaunches, versionLaunches, session, sampling, instance, table.toString());
  });
}

function asyncUsersData(osLaunches, versionLaunches, session, sampling, instance, update, callback) {
  var str = '';
  var mainOS = ['Total', 'Mac', 'Windows', 'Linux', 'undefined'];
  var p = usageReport.getActiveUsers();
  p.then(function(data) {
    for (var key in data) {
      if (data.hasOwnProperty(key)) {
        if (!helpers.isInArray(key, mainOS)) {
          str += '\t';
        }
        str += chalk.green.underline.underline(key) + ': ' + data[key] + '\n';
      }
    }
    callback(null, osLaunches, versionLaunches, session, sampling, instance, update, str);
  }, function(reason) {
    console.log(reason);
  });
}

async.waterfall([
  asyncOsLaunches,
  asyncVersionLaunches,
  asyncSessionData,
  asyncSamplingData,
  asyncInstanceData,
  asyncUpdateData,
  asyncUsersData,
  function(osLaunches, versionLaunches, session, sampling, instance, update, users) {
    var reports = options.report;
    if (helpers.isInArray('launches', reports)) {
      console.log(chalk.inverse.bold('App Launches By OS'));
      console.log(osLaunches);
      console.log();
      console.log(chalk.inverse.bold('App Launches By Compass Version'));
      console.log(versionLaunches);
      console.log();
    }
    if (helpers.isInArray('session', reports)) {
      console.log(chalk.inverse.bold('Session (min)'));
      console.log(session);
      console.log();
    }
    if (helpers.isInArray('sampling', reports)) {
      console.log(chalk.inverse.bold('Sampling (ms)'));
      console.log(sampling);
      console.log();
    }
    if (helpers.isInArray('instance', reports)) {
      console.log(chalk.inverse.bold('Instance Connection Flow'));
      console.log(instance);
      console.log();
    }
    if (helpers.isInArray('update', reports)) {
      console.log(chalk.inverse.bold('Update Connection Successful'));
      console.log(update);
      console.log();
    }
    if (helpers.isInArray('users', reports)) {
      console.log(chalk.inverse.bold('Users'));
      console.log(users);
    }
  }
]);
